/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef CHANNEL4_PROTOCOL_CRYPT_H
#define CHANNEL4_PROTOCOL_CRYPT_H

// Basisklasse einbinden.
#include "Channel4Protocol.h"
#include <time.h>
#include <string>

#include "../Common/crc.h"
#include "../Common/aescryptsession.h"

#define IV_MODULE_PROTOCOL_CRYPT                 0x0000
#define IV_FUNCTION_PROTOCOL_PING                0x0000
#define IV_FUNCTION_EXCHANGE_PUBLIC_KEY          0x0046

/// Basisklasse fuer pure protokollbasierte Kommunikationsaufgaben.
class Channel4ProtocolCrypt : public Channel4Protocol
{
public:
    Channel4ProtocolCrypt( short channelId = IV_DEFAULT_CHANNELID, short deviceId = IV_DEFAULT_DEVICEID );
    virtual ~Channel4ProtocolCrypt();

public:
    /// Oeffnen der Schnittstelle
    int open( enumType type, const std::string &string, int val, int timeout )
    {
        return Channel4Protocol::open( type, string, val, timeout );
    }
    /// Schliessen der Schnittstelle.
    virtual int close()
    {
        return Channel4Protocol::close();
    }

    // Speicher fuer Pakete/Daten
public:
    void reset();

public:
    unsigned char _packetNumberPrevious;
    unsigned char _packetDataPrevious[IV_PACKET_LENGTH_DATA];
    unsigned short _packetDataSizePrevious;

protected:
    AESCryptSession _aesSession[32];
    inline unsigned char getCurrentSessionId()
    {
        return ((getDeviceIdRemote() < 1 || getDeviceIdRemote() > 31) ? 0 : getDeviceIdRemote());
    }
    inline AESCryptSession &getCryptSession()
    {
        return _aesSession[getCurrentSessionId()];
    }

protected:
    // Passwort fuer die verschluesselte Kommunikation setzen
    int setCommunicationPassword( unsigned char *password, int size, int sessionTimeout, int global );

public:
    // Ausfuehren eines Protokollbefehls
    virtual int call( short context, short module, short func, unsigned char *buf, int &count, int contextValue = -1 );

protected:
    // Packetdaten ggf. verschluesseln.
    virtual int writeProtocolCrypt();
    // Streamdaten ggf. verschluesseln.
    int writeStreamCrypt( std::string &buffer );
    // Packetdaten ggf. entschluesseln.
    virtual int readProtocolCrypt();
    // Streamdaten ggf. entschluesseln.
    virtual int readStreamCrypt( std::string &buffer );
    // Pruefen ob aktuell eine Verschluesselte Session aktiv ist.
    bool isCrypt()
    {
        return (getCryptSession().m_cryptMode > 1);
    }

public:
    // Austausch eines Session-Key fuer die verschluesselte Kommunikation
    int exchangeSessionKey( short deviceId );

public:
    // Gibt Modulfehler zurueck.
    virtual void getErrorModule( short module, short error, std::string &text );
    // Gibt Funktionsfehlertext zurueck.
    virtual void getErrorFunction( short module, short func, short error, std::string &text );
};

#endif
