/**
* @file  TimeboyStation.cpp
* @brief Implementierung der Klasse TimeboyStation.
*/
//////////////////////////////////////////////////////////////////////

#include "TimeboyStation.h"
#ifndef _MSC_VER
#include </usr/include/pthread.h>
#endif
#include <memory.h>

// ##################################################################
/**
* @brief Destruktor schliesst die eventuell noch geoeffnete Schnittstelle. 
*
*/
TimeboyStation::~TimeboyStation() 
{
}


//////////////////////////////////////////////////////////////////////
// Members
//////////////////////////////////////////////////////////////////////

//#########################################################
//######################### CRC ###########################
//#########################################################

// ##################################################################
/**
* @brief Erzeugt einen 16bit CRC Wert fuer die uebergebenen Daten.
*/
unsigned short TimeboyStation::crc16(unsigned char *data, unsigned int size, unsigned short crc)
{
	getChannelBase().trace("%s(%i):INFO(3): %s [ 0x%08X, %i, 0x%04X ]\n", __FILE__, __LINE__, __func__, data, size, crc);

	unsigned int i;
	unsigned char j;

	for (i=0; i < size; i++)
	{
		crc = crc ^ data[i];

		for (j=0; j < 8; j++)
		{
			if (crc & 0x1)
			{
				crc >>= 1; 
				crc = crc ^ 0xA001;
			}
			else 
			{
				crc >>= 1;
			}
		}
	}

	return crc;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
unsigned short TimeboyStation::getSlotFirmwareVersion(short deviceID)
{
	unsigned short ver;
	int res, autodetect = 1;

	// 254 kann nur Einzeldocking sein und nutzt MasterIV Protokoll.
	// Bereich pruefen
	if (deviceID < 1 || deviceID >= 254)
	{
		return 0;
	}

	switch (_timeboySlot[deviceID].getVersion()) 
	{
	case 0xFFFF: // Pruefung bereits durchgefuehrt und war nicht erfolgreich.
		break;

	case 0: 
		{ // Ermitteln der FWVersionProtokollversion
			Channel4Base::enumType type;
			std::string commString;
			int commVal, commTO;
			int commValOld = 0;
			getChannelBase().getOpenParams(type, commString, commVal, commTO);

			while (autodetect)
			{
				// Bei Seriell ist es wahrscheinlicher, dass mit 9600bd kommuniziert wird, deswegen zuerst damit Pruefen und dann mit 38400bd.
				if (type == Channel4Base::Serial && commVal != 9600)
				{
					// Schnittstellenbaud anpassen
					commValOld = commVal;
					commVal = 9600;
					getChannelBase().changeStateSerial(getChannelBase()._comm, 9600, commTO);
				}
				else 
				{
					// Wurde Baud per autodetect abgeaendert?
					if (commValOld)
					{
						getChannelBase().changeStateSerial(getChannelBase()._comm, commValOld, commTO);
					}

					autodetect = 0;
				}

				// In der Version 1.25 (08.2007) wurde die Timeboyerkennung per Kommunikationspolling der Docking auf TB vorgenommen.
				// Ab 1.26 (09.2007) wurde die Unterstützung der Timeboyerkennung per Komperatorschaltung integriert.
				// De-/Aktivierung über 0x8000000C (Wert 2 kann dort verwendet werden), abfrage des Zustandes über 0x8000E000.

				if ((res = readBytes(deviceID, MB_SLOT_VERSION, 0, (unsigned char*)&ver, sizeof(ver))) == IV_ERROR_SUCCESS)
				{
					// < 1.31	Paketpuffer ist 45 Byte
					// >= 1.31	Paketpuffer ist 150 Byte
					_timeboySlot[0].setVersion(commVal); // Im 0 Byte die verwendete Baudrate halten.
					_timeboySlot[deviceID].setVersion( binToUIntMSBFirst((unsigned char*)&ver) );
					return _timeboySlot[deviceID].getVersion();
				}
				else
				{
					_timeboySlot[deviceID].setVersion( 0xFFFF );
				}
			}
			break;
		}

	default:
		return _timeboySlot[deviceID].getVersion();
	}

	return 0;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void TimeboyStation::resetSlot(short deviceID)
{
	if (deviceID < 0 || deviceID >= 254)
	{
		return;
	}

	_timeboySlot[deviceID].Reset();
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void TimeboyStation::resetSlots()
{
  unsigned char deviceID;
  for (deviceID = 0; deviceID < 255; deviceID++) {
    _timeboySlot[deviceID].Reset();
  }
}

int TimeboyStation::callDocking(unsigned char *txBuf, unsigned char txCount, unsigned char *rxBuf, unsigned char *rxCount)
{
	int res, nb;
	int rcnt=0,len;

	// Autobaud
	// Stimmt die aktuell Verwendete Baudrate nicht mit der benoetigten ueberein?
	if (getChannelBase().getType() == Channel4Base::Serial && _timeboySlot[0].getVersion() && getChannelBase().getBaudRate() != _timeboySlot[0].getVersion())
	{
		getChannelBase().changeStateSerial(getChannelBase()._comm, _timeboySlot[0].getVersion(), getChannelBase().getTimeout());
	}

	// CRC noch an Paket anfuegen
	*(unsigned short *)&txBuf[txCount] = crc16(txBuf, txCount);
	txCount += sizeof(unsigned short);

	// Empfangsanzahl initialisieren
	*rxCount = 0;

#ifdef TRACE_DOCKING
	getChannelBase().trace("%s(%i):INFO(1): %s.send( %i ) ( %s[ 0x%02X ], Add: 0x%08X  Cnt: %i )\n", __FILE__, __LINE__, __func__, txBuf[0], (txBuf[3] == MB_READ) ? "READ" : "WRITE", txBuf[3], binToULongMSBFirst(&txBuf[4]), (txBuf[3] == MB_READ) ? txBuf[9] : txBuf[2] - 0x0A);
#endif

	// Paketdaten versenden.
	if ((res = getChannelBase().writeData((void*)txBuf, txCount, nb, 1)) != IV_ERROR_SUCCESS)
	{
		getChannelBase().trace("%s(%i):ERROR(1): %s.transmit [ TransmitData fail with: %i ]\n", __FILE__, __LINE__, __func__, res);
		// Neuerkennung notwendig.
		resetSlot(getDeviceIdRemote());
		return res;
	}
	// Alle Zeichen versendet?
	if (nb != (int)txCount)
	{
		return -IV_ERROR_TRANSMIT;
	}

	// Kopf des Paketes empfangen.
	if ((res = getChannelBase().readData((void*)rxBuf, 4, nb, 1)) != IV_ERROR_SUCCESS)
	{
		getChannelBase().trace("%s(%i):ERROR(1): %s [ ReceiveData fail with: %i ]\n", __FILE__, __LINE__, __func__, res);
		// Neuerkennung notwendig.
		resetSlot(getDeviceIdRemote());
		return res;
	}
	// Alle Zeichen empfangen?
	if (nb < 4) 
	{
		getChannelBase().trace("%s(%i):ERROR(1): %s [ %i von %i Byte empfangen ]\n", __FILE__, __LINE__, __func__, nb, 4);
		return -IV_ERROR_RECEIVE;
	}
	rcnt = nb;

	// Busadresse vergleichen
	if (txBuf[0] != rxBuf[0])
	{
		//#########################
		// Bugfix fuer Dockingstation FW Version 1.24
		// Naechstes Byte ist Adresse?
		if (txBuf[0] == rxBuf[1])
		{
			//getChannelBase().Trace("%s(%i):ERROR(1): %s [ Skip wrong start byte {0x%02X} ]\n", __FILE__, __LINE__, __func__, rxBuf[0]);
			rxBuf[0] = rxBuf[1];
			rxBuf[1] = rxBuf[2];
			rxBuf[2] = rxBuf[3];
			rcnt--;
		}
		else
		{
			// Delay bei TCP/IP ueber Com-Server
			if (getChannelBase().getType() == Channel4Base::Socket)
			{
				Sleep( 200 );
			}

			getChannelBase().trace("%s(%i):ERROR(1): %s [ address wrong %i != %i ]\n", __FILE__, __LINE__, __func__, txBuf[0], rxBuf[0]);
#ifdef _DEBUG
			int i; 
			for (i=0; i < nb; i++) 
			{
				getChannelBase().trace("%02X ", rxBuf[i]);
			}
			getChannelBase().trace("\n");
#endif
			getChannelBase().flush();
			return -IV_ERROR_ADDRESS;
		}
	}

	// Solllaenge Paket aus Kopf ermitteln
	len = binToUIntMSBFirst(&rxBuf[1]);
	if (len < 7 || len > 200) 
	{
		// Delay bei TCP/IP ueber Com-Server
		if (getChannelBase().getType() == Channel4Base::Socket)
		{
			Sleep( 200 );
		}

		getChannelBase().trace("%s(%i):ERROR(1): %s [ packet length < 7 || > 200 {%i}]\n", __FILE__, __LINE__, __func__, len);
#ifdef _DEBUG
		int i; 
		for (i=0; i < nb; i++) 
		{
			getChannelBase().trace("%02X ", rxBuf[i]);
		}
		getChannelBase().trace("\n");
#endif
		getChannelBase().flush();
		return -IV_ERROR_PACKETLENGTH;
	}

	// Body des Paketes empfangen.
	if ((res = getChannelBase().readData((void*)&rxBuf[rcnt], len-rcnt, nb, 1)) != IV_ERROR_SUCCESS) 
	{
		getChannelBase().trace("%s(%i):ERROR(1): %s [ ReceiveData fail with: %i ]\n", __FILE__, __LINE__, __func__, res);
		// Neuerkennung notwendig.
		resetSlot(getDeviceIdRemote());
		return res;
	}
	// Alle Zeichen empfangen?
	if (nb < (len-rcnt)) 
	{
		getChannelBase().trace("%s(%i):ERROR(1): %s [ Nachlesen %i von %i Byte empfangen ]\n", __FILE__, __LINE__, __func__, nb, (len-rcnt));
		getChannelBase().flush();
		return -IV_ERROR_RECEIVE;
	}
	rcnt += nb;

	// Alle Zeichen empfangen?
	if (crc16(rxBuf, rcnt)) 
	{
		getChannelBase().trace("%s(%i):ERROR(1): %s [ Paket CRC ]\n", __FILE__, __LINE__, __func__);
		return -IV_ERROR_CRC_PACKET;
	}

	// Handelt es sich um eine Steuerantwort der Dockingstation?
	if (rcnt == 7)
	{ 
#ifdef TRACE_DOCKING
		getChannelBase().trace("%s(%i):INFO(1): %s.receive ( Befehl: 0x%02X, Fehler: 0x%02X )\n", __FILE__, __LINE__, __func__, rxBuf[3], rxBuf[4]);
#endif
	}
	else
	{
#ifdef TRACE_DOCKING
		getChannelBase().trace("%s(%i):INFO(1): %s.receive ( Befehl: 0x%02X, Länge: %i )\n", __FILE__, __LINE__, __func__, rxBuf[3], rcnt);
#endif
	}

	// Zeitstempel der Antwort halten
	_timeboySlot[getDeviceIdRemote()].setReceiveStampSlot( clock() );

	// Anzahl empfangener Bytes
	*rxCount = rcnt;

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
* @brief Prueft die Erreichbarkeit des Kommunikationspartners.
*
* @param deviceID Anzusprechender Kommunikationspartner.
* @param model Erreichtes Timeboymodel.
*/
int TimeboyStation::getTimeboyModel(short deviceID, int &model)
{
	getChannelBase().trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);
	int res = IV_ERROR_SUCCESS;
	unsigned char data[2];
	memset(data, 0, sizeof(data));

	// Ist die schnelle Steck-Erkennung verfuegbar?
	if (_timeboySlot[deviceID].getDetectionState() == 2)
	{
		// Pruefung ob der Slot auch meldet, dass ein Timeboy steckt.
		if ((res = readBytes(deviceID, MB_SLOT_STATE, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			return res;
		}
		if (*(unsigned short*)data != 0x0100)
		{
			// Timeboy ist nicht erreichbar, da nicht gesteckt!
            //			return -IV_ERROR_UNREACHABLE;
            // KORREKTUR (2012_05_14):
            // Da ab der TimeboyIV Version 04.02.02.40 die Ladeabschaltung voll unterstuetzt wird,
            // Kommt es durch die Abfrageroutine zu Fehlinformationen, da die Ladeschaltung meldet,
            // das kein Timeboy steckt obwohl dieses der Fall ist.
		}
	}

	do 
	{
		// Timeboy ansprechen.
		if ((res = readBytes(deviceID, 0, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
		{
			// Wurde ermittelt, dass ein Timeboy steckt? -> Nochmal versuchen.
			if (*(unsigned short*)data == 0x0100)
			{
				*(unsigned short*)data = 0;
				continue;
			}

			// Fehler ausgeben
			return res;
		}
	} while (res != IV_ERROR_SUCCESS);

	if (data[0] == '~' && data[1] == '~')
	{
		model = 4;
	}
	else
	{
		model = 3;
	}

	// Unabhaengig vom Timeboymodel
	// Pruefung auf die Verfuegbarkeit der Timeboyerkennung durchfuehren.
	if (_timeboySlot[deviceID].getVersion() >= 0x0126 && _timeboySlot[deviceID].getDetectionState() == 0)
	{
		if ((res = readBytes(deviceID, MB_SLOT_CHECK, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
		{
			// Fehler ausgeben
			return res;
		}

		if (*(unsigned short*)data == 0x0002)
		{
			// Wird unterstuetzt
			_timeboySlot[deviceID].setDetectionState( 2 );
		}
		else if (*(unsigned short*)data == 0x0001)
		{
			// Pruefung ob der Slot auch meldet, dass ein Timeboy steckt.
			if ((res = readBytes(deviceID, MB_SLOT_STATE, 0, data, 2, 1)) != IV_ERROR_SUCCESS)
			{
				// Fehler ausgeben
				return res;
			}

			if (*(unsigned short*)data == 0x0100)
			{
				// Timeboy wurde erkannt und der Slot meldet, dass ein Timeboy steckt -> Komperatorschaltung bestueckt.
				*(unsigned short*)data = 0x0002;
				//if ((m_result = writeBytes(deviceID, MB_SLOT_CHECK, data, 2, 1)) != IV_ERROR_SUCCESS)
				//{
				//	// Fehler ausgeben
				//	return m_result;
				//}

				_timeboySlot[deviceID].setDetectionState( 2 );
			}
			else
			{
				_timeboySlot[deviceID].setDetectionState( 1 );
			}
		}
		else
		{
			_timeboySlot[deviceID].setDetectionState( 1 );
		}
	}

	return res;
}

// ##################################################################
/**
* @brief Setzt den Zustand der Roten LED der Mehrfachdocking.
*
* @param deviceID Anzusprechender Kommunikationspartner.
*/
int TimeboyStation::setDockingLED(short deviceID, int state)
{
	getChannelBase().trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);
	int res = IV_ERROR_SUCCESS;
	unsigned char data[1];

	if (state != 0)
	{
		data[0] = 0x00;
	}
	else
	{
		data[0] = 0xFF;
	}

	if ((res = writeBytes(deviceID, MB_SLOT_LED, 0, data, 1)) != IV_ERROR_SUCCESS)
	{
		// Fehler ausgeben
		return res;
	}

#ifdef TRACE_DOCKING
	getChannelBase().trace("%s(%i):INFO(1): %s [ Letzter Wechsel vor %0.3f Sekunden ]\n", __FILE__, __LINE__, __func__, float(clock() - _timeboySlot[deviceID].getLEDStamp()) / float(CLOCKS_PER_SEC));
#endif

	// Zustand der LED und Zeitpunkt des wechsels halten
	_timeboySlot[deviceID].setLEDState((state != 0) ? 0x01 : 0xFF);
	_timeboySlot[deviceID].setLEDStamp( clock() );

	return res;
}

// ##################################################################
/**
* @brief Setzt den Kommunikationszustand eines Slots der Mehrfachdocking.
*
* @param deviceID Anzusprechender Kommunikationspartner.
* @param state Zu setzender Kommunikationszustand.
*/
int TimeboyStation::setTimeboySlotState(short deviceID, int state, int commandResult)
{
	(void)commandResult;
	getChannelBase().trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, state);

	if ( isSlot(deviceID) )
	{
		// Ist der geforderte Zustand gleich dem aktuellen?
		if (_timeboySlot[deviceID].getLEDState() == state)
		{
			return IV_ERROR_SUCCESS;
		}

		if (state != 0)
		{
			int model = 0;
			// Es muss ein gueltiges Timeboymodel vorliegen
			if (getTimeboyModel(deviceID, model) != IV_ERROR_SUCCESS)
			{
				return -IV_ERROR_UNREACHABLE;
			}
			if (model != 4)
			{
				return -IV_ERROR_TIMEBOY_I_II_III;
			}
			// Rote LED einschalten
			if(setDockingLED(deviceID, 1) != IV_ERROR_SUCCESS)
			{
				return -IV_ERROR_UNREACHABLE;
			}
		}
		else
		{
			// Rote LED ausschalten
			if(setDockingLED(deviceID, 0) != IV_ERROR_SUCCESS)
			{
				return -IV_ERROR_UNREACHABLE;
			}
		}
	}
	else
	{
		// Bei der angesprochenen ID handelt es sich um keinen Slot einer TimeboyStation.
		return -IV_ERROR_SLOT_NONE;
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
* @brief Entgegennehmen der Verbindung und starten eines Verarbeitungsthreads.
*/
int TimeboyStation::startCommunicationThread() 
{
	getChannelBase().trace("%s(%i):INFO(3): %s [  ]\n", __FILE__, __LINE__, __func__);

	// Hauptthread starten
	if (_communicationThread == 0)
	{	
#ifdef _MSC_VER
		_communicationThread = ::CreateThread(NULL, 0, communicationThread, this, 0, &_threadID);
#else
		pthread_create(&_communicationThread, NULL, communicationThread, this);
#endif

		int i;
		// Hochlauf des Workers pruefen
		for (i=0; (_communicationThread != 0) && (getThreadState() != idle) && (i < 60); i++)
		{
			Sleep(100);
		}
	}

	return (_communicationThread != 0);
}

// ##################################################################
/**
* @brief Beendigungssignal im Workerthread setzen und auf dessen beendigung warten.
*/
void TimeboyStation::stopCommunicationThread()
{
	getChannelBase().trace("%s(%i):INFO(3): %s", __FILE__, __LINE__, __func__);

	// Socket-Thread deaktivieren, wenn aktiv
	if (_communicationThread != 0)
	{
		// WorkerThread fuer Socket beenden.
		setThreadCommand( exit );

		// Thread umschalten
		Sleep(10);

		// Thread Beendigung abwarten (communicationThread stellt m_endSignal auf FALSE)
		int i;
		for (i=0; i < 60 && getThreadState() != end; i++)
		{				
			getChannelBase().trace("%s(%i):INFO(1): %s [ End wait %i ]\n", __FILE__, __LINE__, __func__, i+1);
			Sleep(100);
		}

		// Hartes Schliessen, wenn noch nicht selbst beendet.
		if (_communicationThread)
		{
#ifdef _MSC_VER
			::CloseHandle(_communicationThread);
#else
			pthread_exit(&_communicationThread);
#endif
			_communicationThread = 0;
		}
	}
}

// ##################################################################
/**
* @brief Workerthread fuer die Verarbeitung der Kommunikationsaufgaben.
*/
#ifdef _MSC_VER
unsigned long WINAPI TimeboyStation::communicationThread(LPVOID lpParam)
#else
void *TimeboyStation::communicationThread(void *lpParam)
#endif
{
	if (lpParam == NULL)
	{	
		// Fehlerhafter Parameter
		return 0;
	}

	// Es wird davon ausgegangen, dass der Parameter ein CChannel4 - Objekt ist.
	TimeboyStation &own = *(TimeboyStation*)lpParam;

	own.getChannelBase().trace("%s(%i):INFO(3): %s [ Begin 0x08X ]\n", __FILE__, __LINE__, __func__, lpParam);

	// Threadzustand setzen
	own._threadState = begin;
	own._threadCommand = none;

	// Hauptschleife ausfuehren?
	int run = TRUE;

	// Hauptschleife des Worker
	while ( run )
	{
		switch ( own._threadState )
		{
		case begin:
			// Threadzustand setzen
			own._threadState = idle;
			break;

		case idle: {
			if (own._threadCommand != none)
			{
				// Threadzustand setzen
				own._threadState = executeCommand;
			}
			else
			{
				//// Pruefung ob LED augeschaltet werden muss.
				//if (own.IsFlag(TSF_AUTO_LED))
				//{
				//	for (deviceID=1; deviceID < 255; deviceID++)
				//	{
				//		if (own._timeboySlot[deviceID].getLEDState() && ((clock() - own._timeboySlot[deviceID].getLEDStamp()) > 3))
				//		{
				//			own.setDockingLED(deviceID, 0);
				//		}
				//	}
				//}

				Sleep(10);
			}
			break;}

		case executeCommand: {

			switch (own._threadCommand)
			{
            case none:
                break;

            case exit:
				// Hauptschleife verlassen
				run = FALSE;
				break;

			case write:
				own._commandParam.res = own.threadWriteBytes(own._commandParam.deviceID, own._commandParam.baseAddress, own._commandParam.offset, own._commandParam.buf, own._commandParam.count, own._commandParam.trials);
				break;

			case read:
				own._commandParam.res = own.threadReadBytes(own._commandParam.deviceID, own._commandParam.baseAddress, own._commandParam.offset, own._commandParam.buf, own._commandParam.count, own._commandParam.trials);
				break;
			}

			// Aufgabe abgearbeitet
			own._threadState = idle;
			own._threadCommand = none;
			break;}

		//case none:
		case end:
		case dummy:
			break;
		}
	}

	// Thread wird beendet
	own._communicationThread = 0;
	// Threadzustand setzen
	own._threadState = end;

	own.getChannelBase().trace("%s(%i):INFO(1): %s [ End ]\n", __FILE__, __LINE__, __func__);

#ifndef _MSC_VER
	pthread_exit(NULL);
#endif
	return 0;
}

//#########################################################
//################## Members ##############################
//#########################################################

/*############################################################################*/
//  FUNKTION:    writeBytes
/** Schreibbefehl, um Bytes in den Timeboy zu schreiben.

Autor                 : Michael Wicher            @n
erstellt am           : 02.03.2007                @n

@param[in] address Zieladresse im Timeboy.
@param[in] buf Zeiger auf Daten, die gesendet werden sollen.
@param[in] count Anzahl Bytes zum Senden.
@param[in] slot Steckplatz der Docking [1..10].

@return #TRUE -> Erfolg, #FALSE -> Miszerfolg.

@b Aufgaben:
- Bytes an den Timeboy senden.

@b Aenderungen:
- 17.01.2008 (MW) Neuer Transferbefehl, um die Kommunikation durch einen Mutex
zu schützen.
- 01.02.2008 (MW) Bei allen Befehlen für den Timeboy wird zuvor bei der Docking
angefragt, ob auch ein Timeboy steckt.
Weiterhin wird hier geprüft, ob sich der Task terminieren soll.
Falls kein Timeboy steckt, wird hier auch die Startphase
abgeschloszen, so dasz die Funktion #DOCKINGControl beendet wird.
- 27.03.2008 (MW) Max. fünf Versuche, den Befehl abzusetzen. Dazwischen wird
immer geprüft, ob der Timeboy laut Docking steckt.
- 19.05.2008 (MW) Vor Abarbeitung des Befehls wird geprüft, ob der ausgewählte
Steckplatz der Docking noch erreichbar ist.
Bei Befehlen dirket für die Docking werden nun 2 Versuche
durchgeführt.
Die CRC-Berechnung verschoben.
*/
/*############################################################################*/

int TimeboyStation::writeBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int transmitTrials)
{
	int res = IV_ERROR_SUCCESS;

	// Ueber den Kommunikationsthread kommunizieren?
	if (_communicationThread)
	{
		_commandParam.deviceID = deviceID;
		_commandParam.baseAddress = baseAddress;
		_commandParam.offset = offset;
		_commandParam.buf = buf;
		_commandParam.count = count;
		_commandParam.trials = transmitTrials;

		// Schreibvorgang starten.
		if (setThreadCommand( write ))
		{
			while (getThreadCommand() != none)
			{
				Sleep(10);
			}
		}
		else
		{
			return -IV_ERROR_MODULE_BUSY;
		}

		return _commandParam.res;
	}

	// Direkte Befehlsausfuehrung
	res = threadWriteBytes(deviceID, baseAddress, offset, buf, count, transmitTrials);

	if (res == IV_ERROR_SUCCESS)
	{
		// Nach erfolgreicher Kommunikation automatisch den
		// Kommunikationsthread starten, wenn noch nicht geschehen.
//		startCommunicationThread();
	}

	return res;
}

int TimeboyStation::threadWriteBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int transmitTrials)
{
	static unsigned char pre[MB_SLOT_BUFFER_SIZE_LARGE];

	int res = IV_ERROR_SUCCESS;
	unsigned char current[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned char i;
	unsigned char limit;
	unsigned char transmit[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned short pause;
	unsigned char receive[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned char repeat;
	unsigned char rxCount;
	unsigned char sendPre;
	int executeFails = 0;
	unsigned int address = baseAddress + offset;

	// Remoteadresse setzen.
	setDeviceIdRemote(deviceID);

	unsigned char bufSize = getDataBufferSize(getDeviceIdRemote());

	// prüfe, ob die Busnummer verstellt ist
	//DOCKINGBusnumVerify(slot);

	for (; count;) 
	{
		sendPre = FALSE;

		for (limit=20; limit; limit >>= 1) 
		{
			// bei allen Befehlen für den Timeboy bei der Docking anfragen, dasz auch ein Timeboy steckt
			//if ((((UINT8 *)&address)[3] & 0xF0) == 0x40) {

			//	KERNELCheckForTerminate(DOCKINGStop);

			//	// falls kein Timeboy steckt, macht der weitere Ablauf keinen Sinn und man kann abbrechen
			//	if (DOCKINGCheckTimeboy(slot) != DOCKING_TIMEBOY_PUSHED) {

			//		// Startphase abgeschloszen, jetzt darf auch der IO-Task loslaufen
			//		globalDOCKING.ready = TRUE;

			//		return (FALSE);

			//	}

			//} else {

			//	limit = 1;

			//}

			current[0] = (unsigned char)getDeviceIdRemote();
			current[1] = 0x00;
			current[2] = 0x0A + ((count > bufSize) ? bufSize : count);
			current[3] = 0x22;
			current[4] = ((unsigned char *)&address)[3];
			current[5] = ((unsigned char *)&address)[2];
			current[6] = ((unsigned char *)&address)[1];
			current[7] = ((unsigned char *)&address)[0];

			memcpy((unsigned char *)&current[8], buf, (count > bufSize) ? bufSize : count);

			repeat = limit;

			// je nach Timeboy oder Docking als Empfänger
			if (current[4] == 0x80) 
			{
				// beim Stellen der Busnummer nur ein Versuch
				if ((current[6] == 0x00) && (current[7] == 0x00)) 
				{
					repeat = 1;
					pause = 0;

					// bei allen anderen Anfragen an die Docking zwei Versuche!
				} 
				else 
				{
					repeat = 2;
					pause = 80;
				}
			} 
			else 
			{
				pause = 2000 / repeat;
			}

			for (i=0; i < repeat; i++) 
			{
				//if (i) DEBUGString(TRACE_DEVICE_DOCKING, "repeat write <%u>.", i);

				if (sendPre == TRUE) 
				{
					// lade, falls nötig,  den Vorgänger
					memcpy(transmit, pre, sizeof(transmit));
				} 
				else 
				{
					// aktuelles Paket laden
					memcpy(transmit, current, sizeof(transmit));
				}

				if ((res = callDocking(transmit, transmit[2] - 2, receive, &rxCount)) == IV_ERROR_SUCCESS) 
				{
					// Anzahl fehlversuche zurücksetzen
					executeFails = 0;

					// Paket für Docking?
					if (transmit[4] == 0x80) 
					{
						if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
						{
							break;
						}
					} 
					else // Paket fuer Timeboy
					{
						if (rxCount == 7) 
						{
							//DEBUGString(TRACE_DEVICE_DOCKING, "%02X, %08lX", receive[4], CONVSwapLong(*(UINT32 *)&transmit[4]));

							if (receive[4] == 0x22) 
							{
								//KERNELMilliSleep(100);
								Sleep(100);

								// falls nicht der Vorgänger gesendet wurde
								if (sendPre == FALSE) 
								{
									// gesendetes Paket zur evtl. Wiederholung merken
									memcpy(pre, transmit, sizeof(pre));

									// fertig
									break;
								}

								// Limit neu festlegen
								repeat = limit;

								// jetzt das aktuelle Paket nochmal senden
								sendPre = FALSE;
							} 
							else if ((receive[4] == 0x21) || (receive[4] == 0x24)) 
							{
								//DEBUGString(TRACE_DEVICE_DOCKING, "repeat pre.");

								// falls eine Wiederholung begonnen wird
								if (sendPre == FALSE) 
								{
									// lege Limit neu fest
									repeat = limit;
								}

								sendPre = TRUE;
							}
						}
						else
						{
							//if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
							//{
							//	break;
							//}
						}
					}
				}
				else
				{
					// Fehlerhafte Ausfuehrungen halten
					executeFails++;
					if (executeFails >= transmitTrials)
					{
						return res;
					}
				}

				if (pause) 
				{
					Sleep(pause);
				}
			}

			// bei erfolgreichem Senden keine Wiederholung
			if (i < repeat) 
			{
				break;
			}
		}

		// falls das Limit erreicht wurde, konnte der Befehl nicht erfolgreich gesendet werden!
		if (limit == 0) 
		{
			return -IV_ERROR_UNREACHABLE;
		}

		address += bufSize;
		buf += bufSize;
		count -= (count > bufSize ? bufSize : count);
	}

	// Zeitpunkt der letzten Timeboykommunikation halten
	if ((address & 0x80000000) == 0)
	{
		_timeboySlot[getDeviceIdRemote()].setReceiveStampTimeboy(clock());
	}

	return res;
}

/*############################################################################*/
//  FUNKTION:    readBytes
/** Lesebefehl, um Bytes aus dem Timeboy zu lesen.

Autor                 : Michael Wicher            @n
erstellt am           : 02.03.2007                @n

@param[in] buf Zeiger, wo gelesene Daten hinkopiert werden sollen.
@param[in] address Quelladresse im Timeboy.
@param[in] count Anzahl Bytes zum Lesen.
@param[in] slot Steckplatz der Docking [1..10].

@return #TRUE -> Erfolg, #FALSE -> Miszerfolg.

@b Aufgaben:
- Bytes vom Timeboy lesen.

@b Aenderungen:
- 17.01.2008 (MW) Neuer Transferbefehl, um die Kommunikation durch einen Mutex
zu schützen.
- 01.02.2008 (MW) Bei allen Befehlen für den Timeboy wird zuvor bei der Docking
angefragt, ob auch ein Timeboy steckt.
Weiterhin wird hier geprüft, ob sich der Task terminieren soll.
Falls kein Timeboy steckt, wird hier auch die Startphase
abgeschloszen, so dasz die Funktion #DOCKINGControl beendet wird.
- 27.03.2008 (MW) Max. fünf Versuche, den Befehl abzusetzen. Dazwischen wird
immer geprüft, ob der Timeboy laut Docking steckt.
- 19.05.2008 (MW) Vor Abarbeitung des Befehls wird geprüft, ob der ausgewählte
Steckplatz der Docking noch erreichbar ist.
Bei Befehlen dirket für die Docking werden nun 2 Versuche
durchgeführt.
Die CRC-Berechnung verschoben.
*/
/*############################################################################*/

int TimeboyStation::readBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int receiveTrials)
{
	int res = IV_ERROR_SUCCESS;
	// Ueber den Kommunikationsthread kommunizieren?
	if (_communicationThread)
	{
		_commandParam.deviceID = deviceID;
		_commandParam.baseAddress = baseAddress;
		_commandParam.offset = offset;
		_commandParam.buf = buf;
		_commandParam.count = count;
		_commandParam.trials = receiveTrials;

		// Lesevorgang starten.
		if (setThreadCommand( read ))
		{
			while (getThreadCommand() != none)
			{
				//getChannelBase().Trace("%s(%i):INFO(1): %s [ wait ]\n", __FILE__, __LINE__, __func__);
				Sleep(10);
			}
		}
		else
		{
			return -IV_ERROR_MODULE_BUSY;
		}

		return _commandParam.res;
	}

	// Direkte Befehlsausfuehrung
	res = threadReadBytes(deviceID, baseAddress, offset, buf, count, receiveTrials);

	if (res == IV_ERROR_SUCCESS)
	{
		// Nach erfolgreicher Kommunikation automatisch den
		// Kommunikationsthread starten, wenn noch nicht geschehen.
//		startCommunicationThread();
	}

	return res;
}

int TimeboyStation::threadReadBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int receiveTrials)
{
	int res = IV_ERROR_SUCCESS;
	unsigned char i;
	unsigned char limit;
	unsigned char transmit[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned short pause;
	unsigned char receive[MB_SLOT_BUFFER_SIZE_LARGE];
	unsigned char repeat;
	unsigned char rxCount;
	int executeFails = 0;
	int error21Cnt = 0;
	unsigned int address = baseAddress + offset;

	// Remoteadresse setzen.
	setDeviceIdRemote(deviceID);

	unsigned char bufSize = getDataBufferSize(getDeviceIdRemote());

	if (offset == MB_SLOT_BUFFER_DATA_SIZE_SMALL)
	{
		address = baseAddress;
		if (bufSize == MB_SLOT_BUFFER_DATA_SIZE_SMALL || (count - MB_SLOT_BUFFER_DATA_SIZE_SMALL) <= MB_SLOT_BUFFER_DATA_SIZE_SMALL)
		{
			// Nur wenn der moegliche Lesepuffer groesser ist, macht es sinn die bereits gelesenen Daten nochmals mitzulesen?
			// Die Dockingstation liest ein Packet vor und sendet immer die zuletzt angeforderte Packetgroesse in Folge.
			address += MB_SLOT_BUFFER_DATA_SIZE_SMALL;
			// bereits gelesene Daten von der geforderten Groesse abziehen
			count -= MB_SLOT_BUFFER_DATA_SIZE_SMALL;
			// Puffer weitersetzen
			buf += MB_SLOT_BUFFER_DATA_SIZE_SMALL;
		}
	}

	// prüfe, ob die Busnummer verstellt ist
	//DOCKINGBusnumVerify(slot);

	for (; count;) 
	{
		for (limit=20; limit; limit >>= 1) 
		{
			// bei allen Befehlen für den Timeboy bei der Docking anfragen, dasz auch ein Timeboy steckt
			//if ((((UINT8 *)&address)[3] & 0xF0) == 0x40) {

			//	KERNELCheckForTerminate(DOCKINGStop);

			//	// falls kein Timeboy steckt, macht der weitere Ablauf keinen Sinn und man kann abbrechen
			//	if (DOCKINGCheckTimeboy(slot) != DOCKING_TIMEBOY_PUSHED) {

			//		// Startphase abgeschloszen, jetzt darf auch der IO-Task loslaufen
			//		globalDOCKING.ready = TRUE;

			//		return (FALSE);

			//	}

			//} else {

			//	limit = 1;

			//}

			transmit[0] = (unsigned char)getDeviceIdRemote();
			transmit[1] = 0x00;
			transmit[2] = 0x0C;
			transmit[3] = 0x21;
			transmit[4] = ((unsigned char *)&address)[3];
			transmit[5] = ((unsigned char *)&address)[2];
			transmit[6] = ((unsigned char *)&address)[1];
			transmit[7] = ((unsigned char *)&address)[0];
			transmit[8] = 0x00;
			transmit[9] = (count > bufSize) ? bufSize : count;

			repeat = limit;

			// je nach Timeboy oder Docking als Empfänger
			if (transmit[4] == 0x80) 
			{
				repeat = 2;
				pause = 80;
			} 
			else 
			{
				pause = 2000 / repeat;
			}

			for (i=0; i < repeat; i++) 
			{
				//if (i) DEBUGString(TRACE_DEVICE_DOCKING, "repeat <%u>.", i);

				if ((res = callDocking(transmit, 0x0A, receive, &rxCount)) == IV_ERROR_SUCCESS) 
				{
					// Fehlerhafte Ausfuehrungen zuruecksetzen
					executeFails = 0;

					if (transmit[4] == 0x80)
					{
						// wurde ein gültiges Antwortpaket empfangen, prüfen, ob auch die Länge paszt
						if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
						{
							break;
						}
					}
					else
					{
						if (rxCount == 7)
						{
							switch ( receive[4] )
							{
							case 0x22:
							case 0x23:
								break;
							case 0x21:
							case 0x24:
								{
									// Erster Befehl kann aufgrund von Sleep unbeantwortet bleiben.
									error21Cnt++;
									if (error21Cnt > 1)
									{
										return -IV_ERROR_UNREACHABLE;
									}
								}
								break;
							}
						}
						else
						{
							// Fehlerzaehler ruecksetzen.
							error21Cnt = 0;

							// wurde ein gueltiges Antwortpaket empfangen, pruefen, ob auch die Adresse passt
							if (*(unsigned int *)&receive[4] == *(unsigned int *)&transmit[4]) 
							{
								break;
							}
						}
					}
				}
				else
				{
					// Fehlerhafte Ausfuehrung
					executeFails++;
					if (executeFails >= receiveTrials)
					{
						return res;
					}
				}

				//KERNELMilliSleep(pause);
				Sleep(pause);
			}

			// bei erfolgreichem Senden keine Wiederholung
			if (i < repeat) 
			{
				break;
			}
		}

		// falls das Limit erreicht wurde, konnte der Befehl nicht erfolgreich gesendet werden!
		if (limit == 0) 
		{
			return -IV_ERROR_UNREACHABLE;
		}

		memcpy(buf, (unsigned char *)&receive[8], (count > bufSize) ? bufSize : count);

		buf += bufSize;
		address += bufSize;
		count -= ((count > bufSize) ? bufSize : count);
	}

	// Zeitpunkt der letzten Timeboykommunikation halten
	if ((address & 0x80000000) == 0)
	{
		_timeboySlot[getDeviceIdRemote()].setReceiveStampTimeboy(clock());
	}

	return res;
}

